CASHIER_ENDPOINTS = {
    'sandbox': "https://testapi.opaycheckout.com/api/v1/international/cashier/create",

    'production': "https://api.opaycheckout.com/api/v1/international/cashier/create"
}

PAYMENT_STATUS = {
    'sandbox': "https://sandboxapi.opaycheckout.com/api/v1/international/cashier/status",
    'production': "https://api.opaycheckout.com/api/v1/international/cashier/status"
}

ORDER_EXISTS = "Order already exists"
AUTHENTICATION_FAILED = "authentication failed"
INVALID_PARAMAS = "request parameters not valid"
PAY_METHOD_ERROR = "payMethod not supported"
ALREADY_EXISTS = "the payment reference(merchant order number) already exists."
MERCHANT_NOTCONFIG = "merchant not configured with this function."
MERCHANT_NOT_AVAILABLE = "merchant not available"
SERVICE_NOT_AVAILABLE = "service not available, please try again."



from enum import Enum
from pydantic import BaseModel, EmailStr
from typing import List, Optional
import json
class Amount(BaseModel):
    total: int
    currency: str


class ProductList(BaseModel):
    productId: str
    name: str
    description: str
    price: int
    quantity: int
    imageUrl: Optional[str] = None

class UserModel(BaseModel):
    userId: Optional[str] = None
    userName: Optional[str] = None
    userMobile: Optional[str] = None
    userEmail: Optional[EmailStr] = None

class Product(BaseModel):
    name: Optional[str] = None
    description: Optional[str] = None

class Params(BaseModel):
    reference: str
    country: str
    amount: Amount
    callbackUrl: Optional[str] = None
    returnUrl: str
    productList: List[ProductList]
    cancelUrl: Optional[str] = None
    userClientIP: Optional[str] = None
    expireAt: Optional[int] = None
    userInfo: UserModel
    product: Optional[Product] = None
    payMethod: Optional[str] = None



class Status(Enum):
    INITIAL = "INITIAL"
    PENDING = "PENDING"
    SUCCESS = "SUCCESS"
    FAIL = "FAIL"
    CLOSE = "CLOSE"

class Error(BaseModel):
    code: str
    message: str

class ResponseData(BaseModel):
    reference: str
    orderNo: str
    cashierUrl: str
    status: Status
    amount: Amount
    vat: Amount
class Response(BaseModel):
    code: str
    message: str
    data: ResponseData

class Callbackpayload(BaseModel):
    country: str
    fee: str
    displayedFailure: str
    reference: str
    updated_at: str
    currency: str
    feeCurrency: str
    refunded: bool
    timestamp: str
    amount: str
    transactionId: str
    instrumentType: str
    status: str

class Callback(BaseModel):
    payload: Callbackpayload
    sha512: str
    type: str

# {
#    "payload":{
#       "amount":"49160",
#       "channel":"Web",
#       "country":"NG",
#       "currency":"NGN",
#       "displayedFailure":"",
#       "fee":"737",
#       "feeCurrency":"NGN",
#       "instrumentType":"BankCard",
#       "reference":"10023",
#       "refunded":false,
#       "status":"SUCCESS",
#       "timestamp":"2022-05-07T06:20:46Z",
#       "token":"220507145660712931829",
#       "transactionId":"220507145660712931829",
#       "updated_at":"2022-05-07T07:20:46Z"
#    },
#    "sha512":"9f605d69f04e94172875dc156537071cead060bbcaeaca94a7b8805af9f89611e2fdf6836713c9c90b028ca7e4470b1356e996975f2abc862315aaa9b7f2ae2d",
#    "type":"transaction-status"
# }



import json
#from opay.auth import public_key_signature
from opay.express_checkout.opay_cashier import Opay_Cashier
from opay.express_checkout.models import Params
from pathlib import Path
# # Get the path to the .env file located 3 directories up from the current file
# path = Path(__file__).resolve().parent.parent / '.env'

from dotenv import dotenv_values

# Load as a dictionary (doesn't affect os.environ)
config = dotenv_values()
public_key = config["PUBLIC_KEY"]
merchant_id = config["MERCHANT_ID"]

data = "utils/data.json"
with open(data, mode="r") as file:
    loaded_data = json.load(file)

app = Opay_Cashier()
app.authentication(
    public_key=public_key, 
    merchant_id=merchant_id
)
print(app.request(payload=loaded_data))

#d = Params(**loaded_data)
# da = d.model_dump()
# print(json.dumps(da, indent=4))
##print(app)



import fastapi
from opay.express_checkout.models import Callback

app = fastapi.FastAPI()

@app.post("/callback/")
def webhook_func(data:Callback):
    print ({"message":"sucess", "data": f"{data.model_dump()}"}
)
@app.post("/cancelurl")
def cancel(data:Callback):
    return {"message":"sucess", "data": f"{data}"}

@app.post("/returnurl")
def returnUrl(data:Callback):
    return {"message":"sucess", "data": f"{data}"}





from .models import * 
from .auth import Auth
import utils.constants as constants
from utils.custom_error import Opay_ResponseHandler, Custom_Response
from typing import Optional, Any, Dict
import requests
from requests.exceptions import ConnectionError
import time



class Opay_Cashier:
    def __init__(self, environment: str = "sandbox", auth_keys: Optional[Dict[str, Any]] = None) -> None:
        """
        Initialize the Opay_Cashier instance with the specified environment and optional authentication keys.

        Args:
            environment (str): The operating environment for the Opay_Cashier instance, either "sandbox" or "production".
            auth_keys (Optional[Dict[str, Any]]): Optional dictionary containing authentication credentials.

        Raises:
            ValueError: If the environment is not "sandbox" or "production".
        """

        self.environment = environment
        self.auth_keys = auth_keys if auth_keys else {}
        self.auth = Auth(**self.auth_keys)

        
        # Accessing CASHIER_ENDPOINTS dictionary
        self.base_url = constants.CASHIER_ENDPOINTS.get(self.environment)
        
        if not self.base_url:
            raise ValueError("Invalid Environment: Environment should be 'sandbox' or 'production'")

    def authentication(self, public_key: Optional[str] = None, merchant_id: Optional[str] = None) -> dict:
        """
        Set or generate authentication keys for the Opay_Cashier instance.

        If the auth_keys attribute is empty, this method will validate and set the
        authentication keys using the provided arguments, or generate them
        using the public_key_signature function.

        Args:
            public_key (Optional[str]): The public key for authentication.
            merchant_id (Optional[str]): The merchant ID for authentication.

        Returns:
            dict: A dictionary containing the authentication keys.

        Raises:
            ValueError: If the required 'public_key' and 'merchant_id' are missing.
        """
        # Check if auth_keys are already set, otherwise validate the provided arguments
        if not self.auth_keys:
            # Ensure both 'public_key' and 'merchant_id' are provided
            if public_key and merchant_id:
                self.auth_keys = {
                    "Content-Type": "application/json",
                    "Authorization": f"Bearer {public_key}",
                    "MerchantId": merchant_id
                }
            else:
                # Generate keys using public_key_signature if no arguments are provided, to use this function, it must have been passed in the Opay_Cashier class initalization. 
                self.auth_keys = self.auth.public_key_signature()

                # Verify that the generated headers contain the required fields
                if 'Authorization' not in self.auth_keys or 'MerchantId' not in self.auth_keys:
                    raise ValueError("Authentication failed: Required 'public_key' and 'merchant_id' are missing.")
        print(self.auth_keys)
        return self.auth_keys
    

    # def __repr__(self) -> str:
    #     return (f"Opay_Cashier(environment: {self.environment}, auth_keys: {self.auth_keys}, "
    #             f"base_url: {self.base_url})")

    def request(self, payload: dict) -> dict:
        # Validate and prepare the payload
        self.payload: Params = Params(**payload)  
        self.data = self.payload.model_dump()
        
        # Authenticate and set headers
        self.auth_keys = self.authentication()  # This ensures headers are set or generated correctly

        #To-do: error handling for failed connection
        self.response = requests.post(
        url= self.base_url, json=self.data, headers=self.auth_keys)
        data = self.response.json()
        if data["code"] != "00000":
            error = Error(**data).model_dump()
            error_code = error["code"]
            opay_handler = Opay_ResponseHandler(error_code=error_code)
            # print (opay_handler.response)
            return opay_handler.get_response()

        else:
            #print(data)
            res = Response(**data).model_dump()
            success =Custom_Response()
            response = success.success_response(res)
            print(response)
            
       



from utils.custom_error import Environment_Variable_Exception
import hashlib
import hmac
import os
from dotenv import load_dotenv, find_dotenv, set_key



class Auth:
    def __init__(self, public_key: str = None, merchant_id: str = None, private_key: str = None):
        """
        Initialize the Auth instance with optional public key, merchant ID, and private key.

        Args:
            public_key (str, optional): The public key for authentication. Defaults to None.
            merchant_id (str, optional): The merchant ID for authentication. Defaults to None.
            private_key (str, optional): The private key for generating signatures. Defaults to None.
        """

        self.pub_key = public_key
        self.merchant_id = merchant_id
        self.prv_key = private_key

    def get_env_value(self, var_name: str) -> str | None:
        """
        Retrieves the value of a specified environment variable from the .env file.

        Args:
            var_name (str): The name of the environment variable to retrieve.

        Returns:
            str | None: The value of the specified environment variable, or None if it does not exist.

        Raises:
            Environment_Variable_Exception: If the specified environment variable does not exist.
        """
        load_dotenv(find_dotenv())
        value = os.getenv(var_name)
        if value is None:
            raise Environment_Variable_Exception(
                f"Environment variable '{var_name}' not found. Please check your .env file."
            )
        return value

    def get_public_key(self) -> dict[str, str]:
        """
        Retrieves the public key and merchant ID from the .env file.

        Returns:
            dict[str, str]: A dictionary containing the public key and merchant ID.

        Raises:
            ValueError: If the public key or merchant ID is not found in the .env file.
        """
        if self.pub_key is None:
            self.pub_key = self.get_env_value("PUBLIC_KEY")
        if self.merchant_id is None:
            self.merchant_id = self.get_env_value("MERCHANT_ID")

        if self.pub_key and self.merchant_id:
            return {"pub_key": self.pub_key, "merchant_id": self.merchant_id}
        raise ValueError("Public key or merchant ID not found")

    def public_key_signature(self) -> dict[str, str]:
        """
        Generates a public key signature for authentication.

        If the public key is not already set, this method will retrieve the public key and merchant ID from the .env file.

        Returns:
            dict[str, str]: A dictionary containing the public key signature and merchant ID.
        """
        
        if self.pub_key is None:
            auth_details = self.get_public_key()
            self.pub_key = auth_details["pub_key"]
            self.merchant_id = auth_details["merchant_id"]

        return {
            'Authorization': f'Bearer {self.pub_key}',
            'MerchantId': self.merchant_id,
        }

    def get_private_key(self) -> dict[str, str]:
        """
        Retrieves the private key and merchant ID from the .env file.

        If the private key or merchant ID is not already set, this method will retrieve the private key and merchant ID from the .env file.

        Returns:
            dict[str, str]: A dictionary containing the private key and merchant ID.

        Raises:
            ValueError: If the private key or merchant ID is not found in the .env file.
        """
        if self.prv_key is None:
            self.prv_key = self.get_env_value("PRIVATE_KEY")
        if self.merchant_id is None:
            self.merchant_id = self.get_env_value("MERCHANT_ID")

        if self.prv_key and self.merchant_id:
            return {"prv_key": self.prv_key, "merchant_id": self.merchant_id}
        raise ValueError("Private key or merchant ID not found")

    def private_key_signature(self, payload: str) -> dict[str, str]:
        """
        Generates a signature using the private key.

        If the private key is not already set, this method will retrieve the private key and merchant ID from the .env file.

        Args:
            payload (str): The payload to sign.

        Returns:
            dict[str, str]: A dictionary containing the signature, merchant ID, and content type.

        Raises:
            ValueError: If the private key or merchant ID is not found in the .env file.
        """
        if self.prv_key is None:

            auth_details = self.get_private_key()
            self.prv_key = auth_details["prv_key"]
            self.merchant_id = auth_details["merchant_id"]

        signature = hmac.new(self.prv_key.encode(), payload.encode(), hashlib.sha512).hexdigest()
        return {
            'Authorization': f'Bearer {signature}',
            'MerchantId': self.merchant_id,
            'Content-Type': 'application/json'
        }




# Two auth methods involves passing the key straight into the Opay cashier class or with the auth method in the class 



from typing import Optional
import utils.constants as constants

class Environment_Variable_Exception(Exception):
    def __init__ (self, variable_name):
        self.variable_name = variable_name
        super().__init__(f"Environment_Variable_Error: Variable missing {self.variable_name}")

class Environment_Exception(Exception):
    def __init__(self, environment) -> None:
        self.environment = environment
        super().__init__(f"Wrong Enviroment given: {self.environment}. Environment must either be production or development")


class Custom_Response:
    """Custom response class"""

    def success_response(self, data: dict, status_code: int = 200, code: str = "00000") -> dict:
        return {'status': 'success', 'data': data, "code": code, "status_code": status_code}

    def base_error_response(self, message: str, status_code: int, error_code: str) -> dict:
        return {'status': 'error', 'error': message, "status_code": status_code, "code": error_code}

    def failed_auth(self, message: str = constants.AUTHENTICATION_FAILED) -> dict:
        return self.base_error_response(message, status_code=401, error_code="02000")

    def invalid_params(self, message: str = constants.INVALID_PARAMAS) -> dict:
        return self.base_error_response(message, status_code=400, error_code="02001")

    def pay_method_error(self, message: str = constants.PAY_METHOD_ERROR) -> dict:
        return self.base_error_response(message, status_code=400, error_code="02003")

    def referance_exist(self, message: str = constants.ALREADY_EXISTS) -> dict:
        return self.base_error_response(message, status_code=409, error_code="02004")

    def merchant_not_config(self, message: str = constants.MERCHANT_NOTCONFIG) -> dict:
        return self.base_error_response(message, status_code=409, error_code="02002")

    def merchant_not_found(self, message: str = constants.MERCHANT_NOT_AVAILABLE) -> dict:
        return self.base_error_response(message, status_code=400, error_code="02007")

    def service_unavailable(self, message: str = constants.SERVICE_NOT_AVAILABLE) -> dict:
        return self.base_error_response(message, status_code=500, error_code="50003")
    def order_exists(self, message: str = constants.ORDER_EXISTS) -> dict:
        return self.base_error_response(message, status_code=409, error_code="00001")







class Opay_ResponseHandler(Custom_Response):
    def __init__(self, error_code: str):
        self.error_code = error_code
        self.response = self.get_response()

    def get_response(self) -> dict:
        error_map = {
            "02000": self.failed_auth,
            "02001": self.invalid_params,
            "02003": self.pay_method_error,
            "02004": self.referance_exist,
            "02002": self.merchant_not_config,
            "00001": self.order_exists,
            "02007": self.merchant_not_found,
            "50003": self.service_unavailable,
        }
        return error_map.get(self.error_code, lambda: {"status": "error", "message": "Unknown error"})()



from setuptools import setup, find_packages

setup(
    name="PyOpay",
    version="0.1.0",
    description="A client library for interacting with the Opay API",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    author="Precious Ojogu",
    author_email="nkangprecious26@gmail.com",
    url="https://github.com/Prevz26/Opay-py",
    license="MIT",  # No 'license_files'
    packages=find_packages(),
    install_requires=[
        "requests",
        "pydantic",
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",
)



import hashlib
import hmac
import json

from opay.express_checkout.auth import private_key, public_key

import os
import hmac
import hashlib
from dotenv import load_dotenv

def private_key_signature(payload, secret_key=None, path=None):
    """Generate a signature using the private key."""
    if secret_key is None:
        auth_details = private_key(path=path)  # Pass path to private_key function
        secret_key = auth_details.get("private_key", "key not found")
        merchant_id = auth_details.get("merchant_id", "merchant_id not found")

    signature = hmac.new(secret_key.encode(), payload.encode(), hashlib.sha512).hexdigest()
    headers = {
        'Authorization': f'Bearer {signature}',
        'MerchantId': merchant_id,
        'Content-Type': 'application/json'
    }
    return headers

def public_key_signature(pub_key=None, path=None):
    """Generate headers using the public key."""
    if pub_key is None:
        auth_details = public_key(path=path)  # Pass path to public_key function
        pub_key = auth_details.get("pub_key", "key not found")
        merchant_id = auth_details.get("merchant_id", "merchant_id not found")

    headers = {
        'Authorization': f'Bearer {pub_key}',
        'MerchantId': merchant_id,
    }
    return headers

# secret_key = 'your_secret_key'
# merchant_id = '256612345678901'

# # Payload (data you want to send in the request)
# payload = '{"orderId": "12345", "amount": "1000"}'

# # Generate the signature using HMAC-SHA512
# signature = hmac.new(secret_key.encode(), payload.encode(), hashlib.sha512).hexdigest()

# # Set headers
# headers = {
#     'Authorization': f'Bearer {signature}',
#     'MerchantId': merchant_id,
#     'Content-Type': 'application/json'
# }

# # Send the request
# response = requests.post('https://api.opay.com/cashier/payment-status', headers=headers, data=payload)

# # Print response
# print(response.json())
